package dk.sosi.sts.nemlogin.uuid2cpr;

import dk.nemlog_in.lookupservice.LookupService;
import dk.nemlog_in.lookupservice.LookupService_Service;
import dk.sosi.sts.nemlogin.sts.client.NemLoginStsClient;
import org.apache.commons.pool.BasePoolableObjectFactory;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import org.apache.cxf.message.Message;

import javax.xml.namespace.QName;
import java.util.HashMap;
import java.util.Map;

public class LookupServiceObjectFactory extends BasePoolableObjectFactory<LookupService> {
    private Uuid2CprProperties properties;

    public LookupServiceObjectFactory(Uuid2CprProperties properties) {
        this.properties = properties;
    }

    @Override
    public LookupService makeObject() {
        LookupService client = new LookupService_Service().getSoapBindingLookupService();

        // Set up the client
        Client proxy = ClientProxy.getClient(client);

        proxy.getInInterceptors().add(new LoggingInInterceptor());
        proxy.getOutInterceptors().add(new LoggingOutInterceptor());

        proxy.getRequestContext().put("ws-security.sts.applies-to", properties.getAppliesTo());
        proxy.getRequestContext().put("ws-security.signature.username", properties.getSignatureUsername());
        proxy.getRequestContext().put("ws-security.signature.password", properties.getSignaturePassword());
        proxy.getRequestContext().put("ws-security.signature.properties", properties.getSignaturePropertiesFile());
        proxy.getRequestContext().put("ws-security.asymmetric.signature.algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
        proxy.getRequestContext().put("ws-security.is-bsp-compliant", "false");
        proxy.getRequestContext().put(Message.ENDPOINT_ADDRESS, properties.getLookupServiceUrl());

        // Set up the sts client
        NemLoginStsClient stsClient = new NemLoginStsClient(proxy.getBus());

        Map<String, Object> stsProperties = new HashMap<>();
        stsProperties.put("ws-security.signature.username", properties.getSignatureUsername());
        stsProperties.put("ws-security.signature.password", properties.getSignaturePassword());
        stsProperties.put("ws-security.is-bsp-compliant", "false");
        stsProperties.put("ws-security.signature.properties", properties.getSignaturePropertiesFile());
        stsProperties.put("ws-security.encryption.username", properties.getEncryptionUsername());
        stsProperties.put("ws-security.encryption.properties", properties.getEncryptionPropertiesFile());
        stsProperties.put("ws-security.asymmetric.signature.algorithm", "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");
        stsClient.setProperties(stsProperties);
        stsClient.setWsdlLocation("classpath:sts.wsdl");
        stsClient.setServiceQName(new QName("http://docs.oasis-open.org/ws-sx/ws-trust/200512/", "SecurityTokenService"));
        stsClient.setEndpointQName(new QName("http://docs.oasis-open.org/ws-sx/ws-trust/200512/", "STS_Port"));
        stsClient.setSendRenewing(false);
        stsClient.setSendKeyType(false);
        stsClient.setRequiresEntropy(false);
        stsClient.setTokenType("http://docs.oasis-open.org/wss/oasis-wss-saml-token-profile-1.1#SAMLV2.0");
        stsClient.getRequestContext().put(Message.ENDPOINT_ADDRESS, properties.getSecurityTokenServiceUrl());

        stsClient.getInInterceptors().add(new LoggingInInterceptor());
        stsClient.getOutInterceptors().add(new LoggingOutInterceptor());

        proxy.getRequestContext().put("ws-security.sts.client", stsClient);

        return client;
    }
}
