package dk.sosi.sts.nemlogin.uuid2cpr;

import dk.nemlog_in.lookupservice.LookupService;

import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.CprByPidResponse;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.CprBySubjectSerialNumberResponse;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.PidByCprResponse;
import org.datacontract.schemas._2004._07.nemlogin_eia_contracts_datatransferobjects.RidBySubjectSerialNumberResponse;
import org.apache.commons.pool.ObjectPool;

import java.util.Optional;

public class Uuid2CprClient {
    private ObjectPool<LookupService> clientPool;

    public Uuid2CprClient(ObjectPool<LookupService> clientPool) {
        this.clientPool = clientPool;
    }

    public Optional<String> lookupCprFromUuid(String subjectSerialNumber) throws Exception {
        LookupService client = null;
        try {
            client = clientPool.borrowObject();

            CprBySubjectSerialNumberResponse response = client.subjectSerialNumberCpr(subjectSerialNumber);

            if(response.getCpr() == null || response.getCpr().getValue() == null || response.getCpr().getValue().isEmpty()) {
                return Optional.empty();
            }

            return Optional.of(response.getCpr().getValue());
        }
        finally {
            clientPool.returnObject(client);
        }
    }
    
    public Optional<String> lookupCprFromPid(String pid) throws Exception {
        LookupService client = null;
        try {
            client = clientPool.borrowObject();

            CprByPidResponse response = client.pidCpr(pid);

            if(response.getCpr() == null || response.getCpr().getValue() == null || response.getCpr().getValue().isEmpty()) {
                return Optional.empty();
            }

            return Optional.of(response.getCpr().getValue());
        }
        finally {
            clientPool.returnObject(client);
        }
    }
    
    public Optional<String> lookupRidFromUuid(String subjectSerialNumber) throws Exception {
        LookupService client = null;
        try {
            client = clientPool.borrowObject();

            RidBySubjectSerialNumberResponse response = client.subjectSerialNumberRid(subjectSerialNumber);

            if(response.getRid() == null || response.getRid().getValue() == null || response.getRid().getValue().isEmpty()) {
                return Optional.empty();
            }

            return Optional.of(response.getRid().getValue());
        }
        finally {
            clientPool.returnObject(client);
        }
    }
}
