package dk.nsp.uuid2cpr.proxy.health;

import dk.sosi.seal.vault.CredentialVault;

import java.security.KeyStoreException;
import java.security.cert.Certificate;
import java.security.cert.CertificateExpiredException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.X509Certificate;

public class CertificateHealthChecker implements HealthChecker {
    private CredentialVault vault;
    private String alias;
    private String vaultName;

    public CertificateHealthChecker(String alias, CredentialVault vault, String vaultName) {
        this.alias = alias;
        this.vault = vault;
        this.vaultName = vaultName;
    }

    @Override
    public HealthStatus checkHealth() {
        Certificate certificate = null;
        try {
            certificate = vault.getKeyStore().getCertificate(alias);
        }
        catch(KeyStoreException e) {
            throw new IllegalStateException(String.format("Could not check certificate validity: Certificate not present for alias '%s'. VaultName: %s.", alias, vaultName), e);
        }

        if(!(certificate instanceof X509Certificate)) {
            throw new IllegalStateException(String.format("Could not check certificate validity: Certificate for alias '%s' was not an X509Certificate! VaultName: %s.", alias, vaultName));
        }

        X509Certificate x509Certificate = (X509Certificate) certificate;
        try {
            x509Certificate.checkValidity();
        }
        catch (CertificateExpiredException e) {
            return HealthStatus.unhealthy(String.format("Certificate for alias '%s' is expired. VaultName: %s. Serial number: %d. Expires: %s", alias, vaultName, x509Certificate.getSerialNumber(), x509Certificate.getNotAfter()));
        }
        catch (CertificateNotYetValidException e) {
            return HealthStatus.unhealthy(String.format("Certificate for alias '%s' is not valid yet. VaultName: %s. Serial number: %d. Expires: %s", alias, vaultName, x509Certificate.getSerialNumber(), x509Certificate.getNotAfter()));
        }

        return HealthStatus.ok(String.format("Certificate for alias '%s' OK. VaultName: %s. Serial number: %d. Expires: %s", alias, vaultName, x509Certificate.getSerialNumber(), x509Certificate.getNotAfter()));
    }
}
