package dk.nsp.uuid2cpr.proxy.health;

import java.util.ArrayList;
import java.util.List;

public class CompositeHealthChecker implements HealthChecker {
    List<HealthChecker> healthCheckers;

    public CompositeHealthChecker(List<HealthChecker> healthCheckers) {
        this.healthCheckers = healthCheckers;
    }

    @Override
    public HealthStatus checkHealth() {
        if(healthCheckers == null) {
            throw new IllegalArgumentException("HealthChecker-list was null!");
        }

        boolean healthy = true;
        List<String> messages = new ArrayList<>();
        for(HealthChecker checker : healthCheckers) {
            HealthStatus status = checker.checkHealth();

            healthy = healthy && status.isHealthy();
            messages.addAll(status.getMessages());
        }

        return healthy ? HealthStatus.ok(messages) : HealthStatus.unhealthy(messages);
    }
}
