package dk.nsp.uuid2cpr.proxy.ws;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.api.SLALogger;
import org.apache.log4j.Logger;
import org.openapitools.api.ServiceApi;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import java.util.Optional;

import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprClient;

public class ServiceApiImpl implements ServiceApi {
	private final static Logger logger = Logger.getLogger(ServiceApiImpl.class);

    private static final String SLA_LOGPOINT_ID = "1";
    private static final String SLA_LOGPOINT_NAME = "Uuid2Cpr.lookup";

    private SLALogger slaLogger;

    private Uuid2CprClient client;

    private String lookupServiceUrl;

    public ServiceApiImpl(SLALogger slaLogger, Uuid2CprClient client, String lookupServiceUrl) {
        this.slaLogger = slaLogger;
        this.client = client;
        this.lookupServiceUrl = lookupServiceUrl;
    }

    @Override
    public Response serviceLookupUuidGet(String uuid, SecurityContext securityContext) {
        logger.info(String.format("Invoking uuid2cpr with uuid %s", uuid));

        SLALogItem item = createSLALogItem("lookup", "uuid", uuid);
        Optional<String> cpr = Optional.empty();
        try {
            cpr = client.lookupCprFromUuid(uuid);
            item.setCallResultOk();
        }
        catch(Exception e) {
            logger.error(String.format("Error invoking uuid2cpr-service with uuid %s", uuid), e);
            item.setCallResultError(e.getMessage());
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(String.format("An error occurred looking up cpr for uuid %s. Please try again later.", uuid)).build();
        }
        finally {
            item.store();
        }

        if(cpr.isPresent()) {
            return Response.ok(ModelMapper.mapToResponseWithDetails(cpr.get())).build();
        }
        return Response.status(Response.Status.NOT_FOUND).entity(String.format("Uuid %s has no associated cpr.", uuid)).build();
    }
    
    @Override
    public Response serviceLookupPidCprPidGet(String pid, SecurityContext securityContext) {
        logger.info(String.format("Invoking uuid2rid service with pid %s", pid));

        SLALogItem item = createSLALogItem("lookup-pid-cpr", "pid", pid);
        Optional<String> cpr = Optional.empty();
        try {
            cpr = client.lookupCprFromPid(pid);
            item.setCallResultOk();
        }
        catch(Exception e) {
            logger.error(String.format("Error invoking pid2cpr-service with pid %s", pid), e);
            item.setCallResultError(e.getMessage());
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(String.format("An error occurred looking up cpr for pid %s. Please try again later.", pid)).build();
        }
        finally {
            item.store();
        }

        if(cpr.isPresent()) {
            return Response.ok(ModelMapper.mapToResponseWithDetails(cpr.get())).build();
        }
        return Response.status(Response.Status.NOT_FOUND).entity(String.format("Pid %s has no associated cpr.", pid)).build();
    }
    
    @Override
    public Response serviceLookupUuidRidUuidGet(String uuid, SecurityContext securityContext) {
        logger.info(String.format("Invoking uuid2rid service with uuid %s", uuid));

        SLALogItem item = createSLALogItem("lookup-uuid-rid", "uuid", uuid);
        Optional<String> rid = Optional.empty();
        try {
            rid = client.lookupRidFromUuid(uuid);
            item.setCallResultOk();
        }
        catch(Exception e) {
            logger.error(String.format("Error invoking uuid2rid-service with uuid %s", uuid), e);
            item.setCallResultError(e.getMessage());
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).entity(String.format("An error occurred looking up rid for uuid %s. Please try again later.", uuid)).build();
        }
        finally {
            item.store();
        }

        if(rid.isPresent()) {
            return Response.ok(ModelMapper.mapToResponseWithDetails(rid.get())).build();
        }
        return Response.status(Response.Status.NOT_FOUND).entity(String.format("Uuid %s has no associated rid.", uuid)).build();
    }

    private SLALogItem createSLALogItem(String path, String keyLabel, String keyValue) {
        SLALogItem item = slaLogger.createLogItem(SLA_LOGPOINT_NAME, SLA_LOGPOINT_ID);
        item.addCallParameter("lookupServiceUrl", lookupServiceUrl);
        item.addCallParameter("lookupServicePath", path);
        item.addCallParameter(keyLabel, keyValue);
        return item;
    }
}
