package dk.nsp.uuid2cpr.proxy.health;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.util.Arrays;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class CompositeHealthCheckerTest {
    private CompositeHealthChecker subject;

    @Test(expected = IllegalArgumentException.class)
    public void checkHealth_null_throwsException() {
        // Arrange
        subject = new CompositeHealthChecker(null);

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
    }

    @Test
    public void checkHealth_emptyList_returnsOK() {
        // Arrange
        subject = new CompositeHealthChecker(Arrays.asList());

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
        assertTrue(result.isHealthy());
        assertTrue(result.getMessages().isEmpty());
    }

    @Test
    public void checkHealth_singleChecker_reportsResultFromComponent() {
        // Arrange
        HealthChecker checker = Mockito.mock(HealthChecker.class);
        Mockito.when(checker.checkHealth()).thenReturn(HealthStatus.ok("OK"));

        subject = new CompositeHealthChecker(Arrays.asList(checker));

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
        assertTrue(result.isHealthy());
        assertTrue(result.getMessages().contains("OK"));
    }

    @Test
    public void checkHealth_multipleCheckers_reportsConjunctionOfResults() {
        // Arrange
        HealthChecker firstChecker = Mockito.mock(HealthChecker.class);
        Mockito.when(firstChecker.checkHealth()).thenReturn(HealthStatus.ok("OK"));

        HealthChecker secondChecker = Mockito.mock(HealthChecker.class);
        Mockito.when(secondChecker.checkHealth()).thenReturn(HealthStatus.unhealthy("unhealthy"));

        subject = new CompositeHealthChecker(Arrays.asList(firstChecker, secondChecker));

        // Act
        HealthStatus result = subject.checkHealth();

        // Assert
        assertFalse(result.isHealthy());
        assertTrue(result.getMessages().contains("OK"));
        assertTrue(result.getMessages().contains("unhealthy"));
    }
}