package dk.nsp.uuid2cpr.proxy.ws;

import dk.sdsd.nsp.slalog.api.SLALogItem;
import dk.sdsd.nsp.slalog.api.SLALogger;
import dk.sosi.sts.nemlogin.uuid2cpr.Uuid2CprClient;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;
import org.openapitools.model.Cpr;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;

import java.io.IOException;
import java.util.Optional;

import static org.junit.Assert.*;

@RunWith(MockitoJUnitRunner.class)
public class ServiceApiImplTest {
    private ServiceApiImpl subject;

    @Mock
    private SLALogger slaLogger;

    @Mock
    private Uuid2CprClient client;

    private static final String LOOKUP_SERVICE_URL = "http://foo";

    @Before
    public void setup() {
        Mockito.when(slaLogger.createLogItem(Mockito.anyString(), Mockito.anyString())).thenReturn(Mockito.mock(SLALogItem.class));

        subject = new ServiceApiImpl(slaLogger, client, LOOKUP_SERVICE_URL);
    }

    // uuid2cpr *******************************************************************************************************
    
    @Test
    public void serviceLookupUuidGet_cprPresent_success() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        String cpr = "0101010101";
        Mockito.when(client.lookupCprFromUuid(uuid)).thenReturn(Optional.of(cpr));

        // Act
        Response result = subject.serviceLookupUuidGet(uuid, context);

        // Assert
        assertEquals(200, result.getStatus());
        assertEquals(cpr, ((Cpr) result.getEntity()).getValue());
    }

    @Test
    public void serviceLookupUuidGet_cprMissing_notFound() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupCprFromUuid(uuid)).thenReturn(Optional.empty());

        // Act
        Response result = subject.serviceLookupUuidGet(uuid, context);

        // Assert
        assertEquals(404, result.getStatus());
    }

    @Test
    public void serviceLookupUuidGet_lookupFails_internalServiceError() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupCprFromUuid(uuid)).thenThrow(IOException.class);

        // Act
        Response result = subject.serviceLookupUuidGet(uuid, context);

        // Assert
        assertEquals(500, result.getStatus());
    }
    
    // pid2cpr *******************************************************************************************************
    
    @Test
    public void serviceLookupPidCprPidGet_cprPresent_success() throws Exception {
        // Arrange
        String pid = "9208-2002-2-130462414956";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        String cpr = "0101010101";
        Mockito.when(client.lookupCprFromPid(pid)).thenReturn(Optional.of(cpr));

        // Act
        Response result = subject.serviceLookupPidCprPidGet(pid, context);

        // Assert
        assertEquals(200, result.getStatus());
        assertEquals(cpr, ((Cpr) result.getEntity()).getValue());
    }

    @Test
    public void serviceLookupPidCprPidGet_cprMissing_notFound() throws Exception {
        // Arrange
    	String pid = "9208-2002-2-130462414956";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupCprFromPid(pid)).thenReturn(Optional.empty());

        // Act
        Response result = subject.serviceLookupPidCprPidGet(pid, context);

        // Assert
        assertEquals(404, result.getStatus());
    }

    @Test
    public void serviceLookupPidCprPidGet_lookupFails_internalServiceError() throws Exception {
        // Arrange
    	String pid = "9208-2002-2-130462414956";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupCprFromPid(pid)).thenThrow(IOException.class);

        // Act
        Response result = subject.serviceLookupPidCprPidGet(pid, context);

        // Assert
        assertEquals(500, result.getStatus());
    }
    
    // uuid2rid *******************************************************************************************************
    
    @Test
    public void sserviceLookupUuidRidUuidGet_ridPresent_success() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        String rid = "19822376";
        Mockito.when(client.lookupRidFromUuid(uuid)).thenReturn(Optional.of(rid));

        // Act
        Response result = subject.serviceLookupUuidRidUuidGet(uuid, context);

        // Assert
        assertEquals(200, result.getStatus());
        assertEquals(rid, ((Cpr) result.getEntity()).getValue());
    }

    @Test
    public void serviceLookupUuidRidUuidGet_ridMissing_notFound() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupRidFromUuid(uuid)).thenReturn(Optional.empty());

        // Act
        Response result = subject.serviceLookupUuidRidUuidGet(uuid, context);

        // Assert
        assertEquals(404, result.getStatus());
    }

    @Test
    public void serviceLookupUuidRidUuidGet_lookupFails_internalServiceError() throws Exception {
        // Arrange
        String uuid = "UI:DK-E:G:00000000-0000-0000-0000-000000000000";
        SecurityContext context = Mockito.mock(SecurityContext.class);

        Mockito.when(client.lookupRidFromUuid(uuid)).thenThrow(IOException.class);

        // Act
        Response result = subject.serviceLookupUuidRidUuidGet(uuid, context);

        // Assert
        assertEquals(500, result.getStatus());
    }
    

    
}